Package: fgga
Type: Package
Title: Hierarchical ensemble method based on factor graph
Version: 1.18.0
Date: 2023-11-27
Authors@R: c(person("Flavio", "Spetale", email = "spetale@cifasis-conicet.gov.ar", role = c("aut", "cre")))
biocViews: Software, StatisticalMethod, Classification, Network,
        NetworkInference, SupportVectorMachine, GraphAndNetwork, GO
Description: Package that implements the FGGA algorithm. This package provides a hierarchical ensemble method based ob factor graphs for the consistent cross-ontology annotation of protein coding genes. FGGA embodies elements of predicate logic, communication theory, supervised learning and inference in graphical models.
Depends: R (>= 4.3), RBGL
Imports: graph, stats, e1071, methods, gRbase, jsonlite, BiocFileCache,
        curl, igraph
Suggests: knitr, rmarkdown, GOstats, GO.db, BiocGenerics, pROC, RUnit,
        BiocStyle
License: GPL-3
URL: https://github.com/fspetale/fgga
Encoding: UTF-8
LazyData: false
VignetteBuilder: knitr
NeedsCompilation: no
git_url: https://git.bioconductor.org/packages/fgga
git_branch: RELEASE_3_22
git_last_commit: 2621333
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Packaged: 2025-10-30 03:57:49 UTC; biocbuild
Author: Flavio Spetale [aut, cre]
Maintainer: Flavio Spetale <spetale@cifasis-conicet.gov.ar>
