export(fgga)
export(fgga2bipartite)
export(svmTrain)
export(svmOnto)
export(msgFGGA)
export(tableTPG)
export(preCoreFG)
export(varianceOnto)
export(createFolds)
export(maxDistancegraphOnto)
export(fHierarchicalMeasures)
export(fMeasures)
export(fMeasuresByLevel)

import("RBGL")
import("stats")
import("BiocFileCache")
importFrom("jsonlite", fromJSON)
importFrom("curl", has_internet)
importFrom("stats", "na.omit", "predict", "setNames")
importFrom("methods", "as", "is")
importFrom("graph", subGraph)
importFrom("gRbase", coerceGraph)
importFrom("gRbase", graph_as)
importFrom("igraph", graph_from_graphnel)
importFrom("gRbase", ancestralSet)
importFrom("gRbase", parents)
importFrom("gRbase", children)
importFrom("e1071", "svm")
importFrom("graph", leaves)
importFrom("graph", nodes)
importFrom("graph", graphNEL)
